#pragma once
/*!
 * \class RichEditEx
 *
 * \brief 
 *
 * \author Alexandr Levchuk
 * \date  2019
 */
/*************************************************************************
The EN_MSGFILTER message notifies a rich edit control's parent window of a
keyboard or mouse event in the control.
        RichEdit.
     - .  
   ,  . 
      .
    ENM_xx.

	NOTIFY_CODE_HANDLER_EX(EN_MSGFILTER, OnEnMsgFilter)
	LRESULT OnEnMsgFilter(LPNMHDR pnmh);
	LRESULT CMainDlg::OnEnMsgFilter(LPNMHDR pnmh)
	{
		if (lpMsgFilter->nmhdr.idFrom == IDC_RICHED1)
		{
			if (lpMsgFilter->msg == WM_KEYDOWN)
			{
				SHORT n; // 0xff80
				if ((n = GetKeyState(VK_CONTROL)) < 0 && lpMsgFilter->wParam == 'V')
				{
					//     unicode.   
					//       
					wstring ws;
					if (edGetClipboardUnicode(ws)) //     unicode.
					{
						edSetClipboardUnicode(ws); //    unicode.
					}
				}
			}
		}
		return 0;
	}

EN_REQUESTRESIZE message notifies a rich edit control's parent window that
the control's contents are either smaller or larger than the control's window size.
  
  _tWinMain  
 HINSTANCE hInstRich = ::LoadLibrary(CRichEditCtrl::GetLibraryName());
 ::FreeLibrary(hInstRich);
*************************************************************************/
/*********************************************************************************
Note 
	WM_NEXTDLGCTL. Sent to a dialog box procedure to set the keyboard focus to a 
	different control in the dialog box. This message performs additional dialog box 
	management operations beyond those performed by the SetFocus function WM_NEXTDLGCTL 
	updates the default pushbutton border, sets the default control identifier, and 
	automatically selects the text of an edit control (if the target window is an
	edit control).
	Do not use the SendMessage function to send a WM_NEXTDLGCTL message if your 
	application will concurrently process other messages that set the focus. Use 
	the PostMessage function instead.

	Called for a control so the control can process arrow-key and TAB-key input itself.
	The WM_GETDLGCODE message is sent to the window procedure associated 
	with a control. By default, the system handles all keyboard input 
	to the control; the system interprets certain types of keyboard input 
	as dialog box navigation keys. To override this default behavior, 
	the control can respond to the WM_GETDLGCODE message to indicate the 
	types of input it wants to process itself.
	
	LRESULT OnGetDlgCode(LPMSG lpmsg)
	{
		if(lpmsg !=0 &&  lpmsg->wParam == VK_RETURN)
			return DLGC_WANTALLKEYS;  // All keyboard input
		if(lpmsg !=0 &&  lpmsg->wParam == VK_LEFT)
			 return DLGC_WANTARROWS;
		return DefWindowProc();
	}
*********************************************************************************/

class RichEditEx:public CWindowImpl<RichEditEx,CRichEditCtrl>
{
public:
	DECLARE_WND_SUPERCLASS(NULL,CRichEditCtrl::GetWndClassName())

	RichEditEx(void) :m_khaki(RGB(189, 183, 107)), m_tomato(RGB(255, 99, 71)), m_green(RGB(0, 205, 102))
		,m_blue(RGB(135, 206, 250))
	{ 
		/*     RichEdit,    
		    (RichEditEx) 
		    ENM_xx. */
		m_maskEvent = ENM_NONE; /* ENM_KEYEVENTS | ENM_SCROLLEVENTS | ENM_MOUSEEVENTS;*/
	};
	
	virtual ~RichEditEx(void) {};

	//LOGFONT m_lf;
	CFont m_font;
	COLORREF m_khaki,m_tomato,m_green,m_blue;
	DWORD m_maskEvent;

	BEGIN_MSG_MAP(RichEditEx)
		MESSAGE_HANDLER_EX(WM_KEYDOWN, OnKeyDown)
		//  WM_GETDLGCODE       
		//        , 
		//    
		//MSG_WM_GETDLGCODE(OnGetDlgCode)	 
	END_MSG_MAP()

	/***********************************************************************
	 FullName:  RichEditEx::OnKeyDown
	 Returns:   LRESULT
	 Description:VK_RETURN, VK_TAB     edit.
		 want return  VK_RETURN.
	 Parameter: UINT uMsg
	 Parameter: WPARAM wParam
	 Parameter: LPARAM lParam
	************************************************************************/
	LRESULT OnKeyDown(UINT uMsg, WPARAM wParam, LPARAM lParam)
	{
		SetMsgHandled(FALSE); // TRUE   
		SHORT n; // 0xff80
		if ((n = GetKeyState(VK_CONTROL)) < 0 && wParam == 'V')
		{ // Ctrl+V
			//     unicode.   
			//       
			wstring ws;
			if (edGetClipboardUnicode(ws)) //     unicode.
			{
				edSetClipboardUnicode(ws); //    unicode.
			}
		}

/*********************************************************************************
Note     
		switch (wParam)
		{
		case VK_RETURN:
		case VK_TAB:
			return 0;
		default: break;
		}
*********************************************************************************/
		return 0; 
	}

	void edInitDefault()
	{
		SetEventMask(GetEventMask() | m_maskEvent);
		long style = ::GetWindowLong(this->m_hWnd,GWL_STYLE);
		ATLASSERT(L"Control needs an ES_MULTILINE style setting" && (style & ES_MULTILINE) == ES_MULTILINE);
		edCreateDefLogFont(9);
		SetBackgroundColor(RGB(0,0,0));
		edSetTextColor(m_green);
		SetSelAll();
		ReplaceSel(0);
	}

	void edInit(COLORREF clrBk, COLORREF clrText = RGB(0, 0, 0), DWORD sizeFont=9)
	{
		SetEventMask(GetEventMask() | m_maskEvent);
		long style = ::GetWindowLong(this->m_hWnd,GWL_STYLE);
		ATLASSERT(L"Control needs an ES_MULTILINE style setting" && (style & ES_MULTILINE) == ES_MULTILINE);
		edCreateDefLogFont(sizeFont);
		SetBackgroundColor(clrBk);
		edSetTextColor(clrText);
		SetSelAll();
		ReplaceSel(0);
	}

private:   
	void edCreateDefLogFont(int sizeFont, int charSet = 0)
	{
		LOGFONT m_lf;
		memset(&m_lf,0,sizeof(LOGFONT));
		wcscpy_s(m_lf.lfFaceName,L"Courier New");
		//  
		m_lf.lfHeight = -MulDiv(sizeFont,GetDeviceCaps(GetDC(),LOGPIXELSY),72);
		if(charSet == 0)  
			m_lf.lfCharSet = RUSSIAN_CHARSET; //   204(0xcc)
		else m_lf.lfCharSet = charSet;
		m_font.CreateFontIndirect(&m_lf);  //  
		SetFont(m_font);
	}

	/***********************************************************
	* Method:    setSelColor
	* Description      
	* Returns:   void
	* Parameter: COLORREF clr
	************************************************************/
	void edSetSelTextColor(COLORREF clr)
	{
		CHARFORMAT2 fmt2;
		fmt2.cbSize = sizeof(CHARFORMAT2);
		fmt2.dwMask |= CFM_COLOR;
		GetSelectionCharFormat(fmt2);
		fmt2.crTextColor = clr;
		//   
		fmt2.dwEffects = CFE_AUTOBACKCOLOR;
		SetCharFormat(fmt2, SCF_SELECTION | SCF_WORD); // 
	}

	/***********************************************************
	* Method:    edStrToLex
	* Description     ,  <, ;>
	* Returns:   void
	* Parameter: const wstring sIn
	* Parameter: vector<wstring> & vsOut
	************************************************************/
	void edStrToLex( const wstring sIn,  vector<wstring> & vsOut)
	{	
		const WCHAR delim[] = {L", ;"};
		WCHAR* token=0;
		WCHAR* cnx =0;

		token = wcstok_s((WCHAR*)sIn.c_str(),delim,&cnx);
		while(token != 0)
		{
			wstring ws(token);
			vsOut.push_back(ws);
			token = wcstok_s( NULL, delim,&cnx ); 
		}
	}

public:
	/***********************************************************
	* Method:    edEditPaste
	* Description      unicode   . 
		       
	*	  .
	* Returns:   BOOL WINAPI
	* Parameter: wstring & ws
	************************************************************/
	BOOL WINAPI edGetClipboardUnicode(wstring &ws)
	{
		BOOL ret = FALSE;
		HGLOBAL hglb;

		// Determines whether the clipboard contains data in the specified format.
		if ( !IsClipboardFormatAvailable(CF_UNICODETEXT)) 
			return FALSE;
		if (!OpenClipboard())
			return FALSE;

		hglb = GetClipboardData(CF_UNICODETEXT); //   unicode	
		if (hglb != NULL)
		{
			LPTSTR lptstr = (LPTSTR)GlobalLock(hglb);
			if (lptstr != NULL)
			{
				ws = lptstr; // return text
				GlobalUnlock(hglb);
				EmptyClipboard();
				ret = TRUE;
			}
		} 
		CloseClipboard();
		return ret;
	}

	/***********************************************************************
	 FullName:  RichEditEx::edSetClipboardUnicode
	 Returns:   BOOL WINAPI
	 Description:       
	 Parameter: wstring & ws
	************************************************************************/
	BOOL WINAPI edSetClipboardUnicode(wstring &ws)
	{
		BOOL ret = FALSE;
		HGLOBAL hglb;

		if (!OpenClipboard())
			return FALSE;

		EmptyClipboard();
		// Allocate a global memory object for the text. 
		size_t sz = (ws.size() + 1)* sizeof(WCHAR);
		hglb = GlobalAlloc(GMEM_MOVEABLE, sz );

		if (hglb != NULL)
		{
			// Lock the handle and copy the text to the buffer. 
			memcpy_s(GlobalLock(hglb),sz, ws.c_str(), sz);
			GlobalUnlock(hglb);

			// Place the handle on the clipboard. 
			SetClipboardData(CF_UNICODETEXT,hglb);
			ret = TRUE;
		}
		CloseClipboard();
		return ret;
	}

	void edScrollToBegin()
	{
		CPoint pt(0,0);
		SetScrollPos(&pt);
	}

	void edScrollToLine(int nLine)	// 
	{
 		CPoint pt(0,0);
 		SetScrollPos(&pt);
		LineScroll(nLine);
	}

	/***********************************************************************
	FullName:  edWriteMultilineText
	Returns:   void
	Qualifier:     
	Parameter: const vector<wstring> & vs
	Parameter: BOOL endstr = TRUE -     
	************************************************************************/
	void edWriteMultilineText(const vector<wstring> & vs, BOOL endstr = FALSE)
	{
		if (edIsMultiline() && !vs.empty())
		{
			int sz = vs.size();
			wstring s;
			for (int i = 0; i < sz; ++i)
			{
				s = vs[i];
				size_t idx = s.rfind(L"\r\n");
				if (idx == wstring::npos && endstr) { s += L"\r\n"; }
				ReplaceSel(const_cast<wchar_t*>(s.c_str()));
			}
		}
	}
	 
	/***********************************************************************
	 FullName:  RichEditEx::edReadAllText
	 Returns:   void
	 Description:    multiline    
	 Parameter: vector<wstring> & vstr
	 Parameter: BOOL del = TRUE -     
	 Parameter: BOOL endstr = TRUE -     ,
		  ,    
	************************************************************************/
	void edReadAllText(vector<wstring>& vstr, BOOL endstr = FALSE, BOOL del = FALSE)
	{
		int num = GetLineCount();
		if (num == 0) return;

		wstring s;
		s.reserve(MAX_PATH);
		for (int i = 0; i < num; ++i)
		{
			int len = LineLength(LineIndex(i));
			s.resize(len + 2);
			int nCh = GetLine(i, const_cast<WCHAR*>(s.c_str()), len);
			size_t idx = s.rfind(L"\r\n");
			if (idx == wstring::npos && endstr) { s += L"\r\n"; }
			if (!del) { vstr.push_back(s); }
			else if (del && nCh == 0)	continue; //   
		}
	}

	/***********************************************************************
	 FullName:  RichEditEx::edGetVbyteFromText
	 Returns:   BOOL
	 Description:       
	 Parameter: vector<BYTE> & vbend
	 Parameter: BOOL endstr = TRUE -     ,
	   ,    
	************************************************************************/
	BOOL edGetVbyteFromText(vector<BYTE> &vbend, BOOL endstr = FALSE)
	{
		if (!edIsMultiline()) return FALSE;

		int num = GetLineCount();
		if (num == 1 && 0 == LineLength(LineIndex(num)) )
			return FALSE;
		
		if (num == 0) return FALSE;

		vbend.clear();
		vector<BYTE> vb;
		wstring ws;
		ws.reserve(MAX_PATH);
		for (int i = 0; i < num; ++i)
		{
			int len = LineLength(LineIndex(i));
			ws.resize(len);
			int nCh = GetLine(i, const_cast<WCHAR*>(ws.c_str()), len);
			size_t idx = ws.rfind(L"\r\n");
			if (idx == wstring::npos && endstr)
			{
				ws += L"\r\n"; nCh += 2;
			}

			int sz = nCh*sizeof(WCHAR);
			vb.resize(sz);
			memcpy_s(&vb[0], sz, &ws[0], sz);
			vbend.insert(vbend.end(), vb.begin(), vb.end());
		}

		if (vbend.empty()) return FALSE;

		return TRUE;
	}

	void edWriteTextFromVbyte(vector<BYTE> &vb)
	{
		if (!edIsMultiline()) return;
		if (vb.empty()) return;

		vector<wstring> vtxt;
		wstring ws;
		size_t sz = vb.size();
		ws.resize(sz / sizeof(WCHAR));
		memcpy_s(&ws[0], sz, &vb[0], sz);
		ReplaceSel(const_cast<wchar_t*>(ws.c_str()));
	}

	/***********************************************************
	* Method:    edWriteLine
	* Description	      
	* Returns:   void
	* Parameter: int nLine
	* Parameter: wstring ws
	* Parameter: COLORREF clr
	************************************************************/
	void edReplaceLine(int nLine, wstring ws)
	{
		wstring s;
		int idx = LineIndex(nLine);
		// read line
		size_t len = LineLength(idx);
		s.resize(len);
		GetLine(nLine,const_cast<WCHAR*>(s.c_str()),len );

		//       , 
		size_t n;
		for(n = 0; n < len; ++n)
		{
			if(s[n] < 0x21)   continue;
			else break;
		}
		idx += n;
		ws += L"\r\n";
		SetSel(idx,idx+s.size()-(n-1));
		ReplaceSel(const_cast<wchar_t*>(ws.c_str()));
	}

	/***********************************************************
	* Method:    edWriteLine
	* Description	       
	* Returns:   void
	* Parameter: int nLine
	* Parameter: wstring ws
	* Parameter: COLORREF clr
	************************************************************/
	void edReplaceLine(int nLine, wstring ws, COLORREF clr)
	{
		wstring s;
		int idx = LineIndex(nLine);
		// read line
		size_t len = LineLength(idx);
		s.resize(len);
		GetLine(nLine,const_cast<WCHAR*>(s.c_str()),len );

		//       
		size_t n;
		for(n = 0; n < len; ++n)
		{
			if(s[n] < 0x21)   continue;
			else break;
		}
		idx += n;
		ws += L"\r\n";
		SetSel(idx,idx+s.size()-(n-1));
		edSetSelTextColor(clr);  // Set color text
		ReplaceSel(const_cast<wchar_t*>(ws.c_str()));
	}

	/***********************************************************
	* Method:    edInsertToLine
	* Description      
	* Returns:   void
	* Parameter: wstring s
	************************************************************/
	void edInsertToLine(wstring s)
	{
		ReplaceSel(const_cast<wchar_t*>(s.c_str()));
	}

	void edInsertToLine(LPCTSTR s)
	{
		ReplaceSel(s);
	}

	void edLineScroll(DWORD nLine = 1)
	{
		LineScroll(nLine);
	}

	//    1
	wstring edReadLine(int nLine=1)
	{
		wstring s;
		int len = LineLength(LineIndex(nLine));
		s.resize(len);
		GetLine(nLine,const_cast<WCHAR*>(s.c_str()),len );

		return s;
	}

	/***********************************************************
	* Method:    edPaintLine
	* Description    
	* Returns:   void
	* Parameter: int nLine
	* Parameter: COLORREF clr
	************************************************************/
	void edPaintLine(int nLine,COLORREF clr)
	{
		wstring s;
		size_t idx = LineIndex(nLine);
		size_t len = LineLength(idx);   
		s.resize(len);
		GetLine(nLine,const_cast<WCHAR*>(s.c_str()),len );   // read line

		//   
		size_t n2 = idx + s.size();
		SetSel(idx,n2);
		edSetSelTextColor(clr);  // Set color text
		ReplaceSel(const_cast<wchar_t*>(s.c_str()));
	}

	/***********************************************************
	* Method:    edPaintSubLine
	* Description        
	* Returns:   void
	* Parameter: int nLine
	* Parameter: wstring ws
	* Parameter: COLORREF clr
	* Parameter: size_t pos -      
	************************************************************/
	void edPaintSubLine(int line, wstring wsSub, COLORREF clr )
	{
		wstring ws;
		size_t idx = LineIndex(line);
		size_t len = LineLength(idx);   
		ws.resize(len);
		GetLine(line,const_cast<WCHAR*>(ws.c_str()),len );   // read line

		//    
		size_t sz = ws.size();
		size_t szSub = wsSub.size();
		for(size_t i = 0,off = 0; i < sz; ++i,off += szSub)	//    
		{
			size_t n1 = ws.find(wsSub,off);
			if(n1 != wstring::npos)
			{
				size_t n2 = (n1+=idx) + wsSub.size();
				//edPaintLine(line,m_blue);

				SetSel(n1,n2);
				edSetSelTextColor(clr);  // Set color text
				ReplaceSel(const_cast<wchar_t*>(wsSub.c_str()));
			}
		}
	}

	/***********************************************************
	* Method:    edPaintWords
	* Description      , 
		  ,    . 
			<, ;>.
	* Returns:   void
	* Parameter: int nLine
	* Parameter: wstring wsSub
	* Parameter: COLORREF clr
	************************************************************/
	void edPaintWords(int nLine, wstring wsSub, COLORREF clr)
	{
		wstring str;
		size_t idx = LineIndex(nLine);
		size_t len = LineLength(idx);   
		str.resize(len);
		GetLine(nLine,const_cast<WCHAR*>(str.c_str()),len );   // read line
		
		//    ,    
		vector<wstring> vws;
		edStrToLex(str,vws); //   
		size_t sz = vws.size();
		
		for(size_t i = 0,off=0; i < sz; ++i)
		{
			if(vws[i] == wsSub)	
			{
				//edPaintLine(line,m_blue);

				size_t n1 = str.find(wsSub,off);
				if(n1 != wstring::npos)
				{
					size_t n2 = (n1+=idx) + wsSub.size();
					SetSel(n1,n2);
					edSetSelTextColor(clr);  // Set color text
					ReplaceSel(const_cast<wchar_t*>(wsSub.c_str())); 
					off += vws[i].size();
				}
			}
		}
	}

	void edClear()
	{
		SetSelAll();
		ReplaceSel(0); 
	}

	void edSetBkClr(COLORREF clr)
	{
		SetBackgroundColor(clr);
	}

	/***********************************************************
	* Method:    edSetColorText
	* Description   set color for all text
	* Returns:   void
	* Parameter: COLORREF clr
	************************************************************/
	void edSetTextColor(COLORREF clr)
	{
		CHARFORMAT2 fmt2;
		fmt2.cbSize = sizeof(CHARFORMAT2);
		GetDefaultCharFormat(fmt2);
		fmt2.dwMask |= (CFM_COLOR | CFM_CHARSET);
		fmt2.crTextColor = clr;
		fmt2.dwEffects = CFE_AUTOBACKCOLOR;	//   
		fmt2.bCharSet = RUSSIAN_CHARSET;
		SetCharFormat(fmt2, SCF_ALL);
	}

	void edSetFont(COLORREF clrText)
	{
		CHARFORMAT cf;
		cf.cbSize = sizeof(CHARFORMAT);
		CRichEditFontDialog fdlg(cf);	 
		if (fdlg.DoModal() == IDOK)
		{
			fdlg.GetCharFormat(cf);
			cf.crTextColor = clrText;
			SetCharFormat(cf,SCF_ALL);
		}
	}

	BOOL edIsMultiline()
	{
		long style = ::GetWindowLong(this->m_hWnd, GWL_STYLE);
		if (style & ES_MULTILINE)
			return TRUE;

		return FALSE;
	}

	/***********************************************************************
	 FullName:  RichEditEx::edGetNumCh
	 Returns:   DWORD
	 Description:    
	 Parameter: vector<wstring> & vws
	************************************************************************/
	DWORD edGetTextLen()
	{
		return GetTextLength();
	}

	DWORD edGetLineCount()
	{
		return GetLineCount();
	}
};


