#pragma once
#include <SetupAPI.h>
#include <Cfgmgr32.h>
#include <Dbt.h>

namespace STM32
{

	/*********************************************************************************
	Note GUID       devguid.h
	#include <devguid.h>
	*********************************************************************************/

	/*********************************************************************************
	Note
	  DBT_DEVICEARRIVAL  DBT_DEVICEREMOVECOMPLETE,   ,
	  guid GUID_DEVINTERFACE_USB_DEVICE,  
	  <usbiodef.h>  DDK    .
	  guid,  ,    DBT_DEVICEARRIVAL 
	  .
	GUID_DEVINTERFACE_USB_DEVICE     const GUID GUID_ = { ....{...}};
	    DDK.
	GUID_DEVCLASS_PORTS        COM  LPT.
	*********************************************************************************/
	const GUID GUID_DEVINTERFACE_USB_DEVICE = { 0xA5DCBF10L, 0x6530, 0x11D2, { 0x90, 0x1F, 0x00, 0xC0, 0x4F, 0xB9, 0x51, 0xED } };
	const GUID GUID_DEVCLASS_PORTS = { 0x4d36e978, 0xe325, 0x11ce, { 0xbf, 0xc1, 0x08, 0x00, 0x2b, 0xe1, 0x03, 0x18 } };

	// This GUID is for all USB serial host PnP drivers, but you can replace it with any valid device class guid.
	// const GUID WceusbshGUID = { 0x25dbce51, 0x6c8f, 0x4a72, {0x8a, 0x6d, 0xb5, 0x4c, 0x2b, 0x4f, 0xc8, 0x35 }};
	// USBDevice includes all USB devices that do not belong to another class.This class is not used for 
	// USB host controllers and hubs.
	// const GUID USBDevice =	  { 0x88BAE032, 0x5A81, 0x49f0, { 0xBC, 0x3D, 0xA4, 0xFF, 0x13, 0x82, 0x16, 0xD6 }};


	/*********************************************************************************
	Note       STM32
	       STM 
	HWND hWndStm = stm.Create(this->m_hWnd,CWindow::rcDefault);
	if (hWndStm != NULL)
	{}
	*********************************************************************************/
#define BAUDRATE	115200
#define SZ_CMD		6	//     
#define SZ_USB_BUF	512 //  
#define SZ_USB_RX	SZ_USB_BUF //    ( STM32)
#define SZ_USB_TX	(SZ_USB_BUF+SZ_CMD) //  +  ,   STM32
#define SZ_PACK_RX			64 //64     STM32
#define OFF_CMD				2 //      
#define OFF_DATA_CMD		4 //    
#define OFF_CMD_EXT			3 //       
#define OFF_DATA_CMD_EXT	5 //        
#define SZ_TIME_RTC			9 //   HAL_RTC 
#define SZ_SET_UX_TIME_RTC	6 //   unix time    
#define SZ_GET_UX_TIME_RTC	8 //   unix time    
#define SZ_TIME_ALARM_RTC	4 //   unix time    
 

	class CSTM32VP : public CWindowImpl<CSTM32VP>
	{
	public:
		DECLARE_WND_CLASS(NULL)

		CSTM32VP();
		~CSTM32VP();

		HWND m_hMain; //  ,   
		static CSTM32VP* pStm;
		wstring m_nameDriverPort;	//     - STMicroelectronics Virtual COM Port (COMxx)

		/*********************************************************************************
		Note:
		,     STM32,     SET  GET. 
		  SET      STM32. 
		 GET    STM32.
		          
		 STM32    ,   .
		
		 cmdSetData -  .
		    STM32   .
		  :
		:
		- byte0, byte1 -   ,
		- byte2 - ID(SET/GET) , 
		- byte3 - ,
		- byte4, byte5 -       STM32.
			 4,5      , ..   .
		- byte0......byteN - ,      .

		STM32          .
		        .  
		 /     .

		cmdGetData -    STM32.
		
		STM32          .
		,     ,   
		    ,    STM32. STM32  
		        .
		   cmdGetData :
		- byte0, byte1 -    =  ,
		
		- byte2 - ID(SET.GET) , 
		- byte3 -  ,
		- byte4 -   
		- byte5 -       (), 
			,  0   .
		*********************************************************************************/
		enum CmdLoEnum :WORD //   STM32
		{
			cmdSetData = 0x0001, //    STM32 
			cmdGetData, //    STM32
		};

		enum CmdHiEnum :WORD
		{
			//     HAL 
			cmdSetDateRTC = cmdSetData | 0x0100,	//   
			cmdSetRTC = cmdSetData | 0x0200,		//     
			cmdSetTimeRTC = cmdSetData | 0x0300,	//   
			cmdGetRTC = cmdGetData | 0x0400,		//  / 
			cmdGetLastAdj = cmdGetData | 0x0500,	// reserve
			
			//      
			cmdSetTmUxRTC = cmdSetData | 0x0600,	//    
			cmdGetTmUxRTC = cmdGetData | 0x0700,	//    
			cmdSetAlarmRTC = cmdSetData | 0x0800,	//    
			cmdGetAlarmRTC = cmdGetData | 0x0900,	//    
			

			cmdSetEcho = cmdSetData | 0xf000,
			cmdGetEcho = cmdGetData | 0xf100,
		};

		enum RTC_Date_Time //    /  RTC
		{
			WeekDay,
			Month,
			Date,
			Year,
			Hours,
			Minutes,
			Seconds,
			Adj0,
			Adj1
		};

		wstring m_wsCom;	// "\\\\.\\COMxx" -   
	private:
		HANDLE m_hCom;
		HANDLE m_hDevNotify;
		vector<BYTE> m_vbTx, m_vbRx; //    

		__time32_t getStrTimeSTM32(wstring & wsTime);
		__time32_t fn_GetStrTimeUxSTM32(wstring & wsTime);
		BOOL receiveData(const WORD cmd, const WORD szData);
		WORD transmitCmdData(const WORD cmd, WORD len = 0);
		BOOL usbWrite(const BYTE* pTx, const WORD len);
		DWORD usbRead(BYTE* pRx);

		void fn_ToLex(const wstring wsIn, const wstring wsDelim, vector<wstring> & vws /*OUT*/);
		WORD fn_MakeTableCalibrateRTC(vector<USHORT> & vcal);
		void fn_ErrorDisplay(wstring & ws, LPTSTR nameFunction = NULL);
	
public:

		void stmClosePort();
		BOOL stmGetTimeRTC(wstring & ws);
		BOOL stmEcho(wstring & ws);
		BOOL stmSetTimeFromHost(wstring & ws, WORD calibr=0);
		vector<BYTE> & stmGetVTx();
		vector<BYTE> & stmGetVRx();
		BOOL stmSearchVirtualComPort(const GUID * guid, vector<wstring> & vws);
		BOOL stmAdjRTC(wstring & nameNTP, wstring & wsMsg);
		BOOL stmAdjUxRTC(wstring & nameNTP, wstring & wsMsg, BOOL noCalibr=0);
		BOOL stmCompareTime(wstring & nameNTP, wstring & wsMsg);
		BOOL stmSetTmAlartmRTC(wstring& wsMsg, BOOL bClearAlarm=0);
		BOOL stmGetTmAlartmRTC(wstring& wsMsg);
		BOOL stmOpenPort(LPCTSTR port);
		BOOL stmIsOpen();

		BOOL stmRegisterNotifyDevChange(GUID guid, HWND wndMain);

		/*********************************************************************************
		Note about message maps with cracked handlers (atlcrack.h):
		You can use BEGIN_MSG_MAP for classes that derive from CWindowImpl/CDialogImpl,
		but must use BEGIN_MSG_MAP_EX for classes that don't.
		*********************************************************************************/
		BEGIN_MSG_MAP_EX(CSTM32VP)
			MESSAGE_HANDLER_EX(WM_USB, OnUsbDevice)
			//MSG_WM_DEVICECHANGE(OnDeviceChange)   , 
		END_MSG_MAP()

		LRESULT OnUsbDevice(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*//*, BOOL& *//*bHandled*/);
	private:

	};
}


