
#ifndef __SNTP_H__
#define __SNTP_H__

/*
* \class CNtpTime
*	     NTP    
*	 NTP,          
*	       SNTP.
*     NTP   DWORD64. 
	  NTP   64-     
	,        1-  1900 .  
	  32 ,     32 . 
	* \author  
*
* \version 1.0
*
* \date  2018
*
* Contact: dslev@yandex.ru
*/
class CNtpTime
{
protected:
	CNtpTime() {};
	~CNtpTime() {};
	/**************************************************************************
	* Method:    getCurrentTime
	* Description::
		     (SYSTEMTIME)   NTP.
	* Returns:   DWORD64
	***************************************************************************/
	DWORD64 getCurrentTime();

	/**************************************************************************
	* Method:    getCurrentTimeForServerNTP
	* Description::
		     SYSTEMTIME  	
		 NTP   ,    (   32  ). 
		  ,    ,   big-endian. 
	* Returns:   DWORD64       NTP
	***************************************************************************/
	DWORD64 getCurrentTimeForServerNTP(DWORD64 & serverTime);
	
	/**************************************************************************
	* Method:    convertNtpTimeToSysTime
	* Description::   NTP     SYSTEMTIME
	* Returns:   void
	* Parameter: DWORD64 ntpTime  ( little endian)
	* Parameter: SYSTEMTIME & st
	***************************************************************************/
	void convertNtpTimeToSysTime(DWORD64 ntpTime,SYSTEMTIME & st /*OUT*/);

	/**************************************************************************
	* Method:    convertNtpTimeToDouble
	* Description::     NTP        
		  .
	* Returns:   double
	* Parameter: const DWORD64 & ntpTime ( little endian)
	***************************************************************************/
	double convertNtpTimeToDouble(const DWORD64 & ntpTime);

	/**************************************************************************
	* Method:    convertNtpTimeToLittleEndian
	* Description:     big-endian  TCP/IP  little-endian  Intel,
		   DWORD32.
	* Returns:   DWORD64
	* Parameter: DWORD64 timeServer (big-endian)
	***************************************************************************/
	DWORD64	convertNtpTimeToLittleEndian (DWORD64 timeServer);

private:
	//  100ns   1601 (FILETIME)  1900 (NTP)
	static const DWORD64 m_1900 = 94354848000000000;  
};							  

/*
* \class CNtpSocket
    NTP   SNTP 
*/
class CNtpSocket: public CNtpTime
{
protected:
	CNtpSocket();
	~CNtpSocket();

	typedef struct _NtpMsg
	{
		DWORD m_LiVnxxx;	// LI, version, mode, stratum, pool,  precision 
		DWORD m_delayRtt;   //    (RTT)    ,  . 
		DWORD m_dispersion;	//           .
		DWORD m_ID;		    //    
		DWORD64 m_ReferenceTime;	//  /  
		DWORD64 m_OriginateTime;	// ,      
		DWORD64 m_ReceiveTime;		// ,         
		DWORD64 m_TransmitTime;		// ,        
	}NtpMsg;

	NtpMsg m_ntpRecive;	//   
	DWORD64 m_sendTime; //    
	static DWORD m_timeout ;	//      

private:
	SOCKET m_hSocket;

	/**************************************************************************
	* Method:    CorrectNtpTime
	* Description:        
		.     .
	* Returns:   DWORD64 -     0.
	***************************************************************************/
	DWORD64 CorrectNtpTime();

protected:

	void CloseSocket();

	/**************************************************************************
	* Method:    ConnectServerNTP
	* Description:
		      addrinfo,     
		-,      bind  connect. 
		   	    connect,  
		    .   socket  connect  ,
		     .  
	* Returns:   BOOL
	* Parameter: LPCSTR pszHostAddress
	* Parameter: int nPort
	***************************************************************************/
	BOOL ConnectServerNTP(LPCSTR sHostAdr, int nPort);

	/**************************************************************************
	* Method:    SendPacketNTP
	* Description:      NTP
	* Returns:   BOOL
	***************************************************************************/
	BOOL SendPacketNTP();

	/**************************************************************************
	* Method:    ReceivePacketNTP
	* Description:         
		    .
	* Returns:   BOOL
	* Parameter: SYSTEMTIME & systm
	***************************************************************************/
	BOOL ReceivePacketNTP(SYSTEMTIME & systm);
};

/*!
* \class CSntpClient
*
*   
*
* \author alex
* \date  2018
*/
class CSntpClient : public CNtpSocket
{
public:
	CSntpClient();
	~CSntpClient();

private:
	int m_port;
	HANDLE  m_hToken;
	TOKEN_PRIVILEGES m_TokenPriv;
	BOOL    m_bTakenPriviledge;

public:
	/**************************************************************************
	* Method:    InitWinSocket
	* Description:   Winsock,    
		 WSACleanup()   Ws2_32.dll.
	* Returns:   BOOL
	***************************************************************************/
	BOOL InitWinSocket();

	/**************************************************************************
	* Method:    CleanUpWinSock
	* Description:   Ws2_32.dll 
	* Returns:   BOOL
	***************************************************************************/
	BOOL CleanUpWinSock();

	/**************************************************************************
	* Method:    GetServerTime
	* Description:    SYSTEMTIME     
					  .
	* Returns:   BOOL = 2 -  ,   
	* Parameter: LPCTSTR pszHostName -   NTP
	* Parameter: SYSTEMTIME & systm
	***************************************************************************/
	BOOL GetServerTime(LPCTSTR pszHostName, SYSTEMTIME & systm);

	/**************************************************************************
	* Method:    SetLocalTime
	* Description:        
	* Returns:   BOOL
	* Parameter: const CNtpTime & NewTime 
	***************************************************************************/
	BOOL SetLocalTime(const SYSTEMTIME & sysNewTm);
	
	/***********************************************************************
	FullName:  TimeEx::FormatSysTimeToWstr
	Returns:   std::wstring
	Description:  /      
	  UTC    
	Parameter: SYSTEMTIME & systm   UTC
	Parameter: BOOL bText
	Parameter: BOOL bLocTime
	************************************************************************/
	std::wstring FormatSysTimeToWstr(SYSTEMTIME& systm /*IN*/, BOOL bText = 0, BOOL bLocTime = 0);

private:
	/**************************************************************************
	* Method:    SetTimeout
	* Description:       .
	* Returns:   void
	* Parameter: DWORD tm
	***************************************************************************/
	void  SetTimeout(DWORD tm=5000);
	DWORD GetTimeout();
	
	/**************************************************************************
	* Method:    IsVerifyReply
	* Description:      .
	* Returns:   BOOL
	***************************************************************************/
	BOOL IsVerifyReply();

	/**************************************************************************
	* Method:    GetStratum
	* Description:     NTP
	* Returns:   DWORD
	***************************************************************************/
	DWORD GetStratum();

	/**************************************************************************
	* Method:    GetPrecision
	* Description:         .
	* Returns:   double
	***************************************************************************/
	double GetPrecision();

	/**************************************************************************
	* Method:    GetPoolInterval
	* Description: -     
		  .
	* Returns:   int
	***************************************************************************/
	int GetPoolInterval();

	/**************************************************************************
	* Method:    GetShiftTime
	* Description:    c       
	* Returns:   void
	* Parameter: double & shiftTime -     
	* Parameter: double & delay	-    
	***************************************************************************/
	void GetShiftTime(double & shiftTime, double & delay);
	

	/**************************************************************************
	* Method:    EnableSetTimePriviledge
	* Description:        
	* Returns:   BOOL
	***************************************************************************/
	BOOL EnableSetTimePriviledge();

	/**************************************************************************
	* Method:    RevertSetTimePriviledge
	* Description:	   
	* Returns:   void
	***************************************************************************/
	void RevertSetTimePriviledge();

	/***********************************************************************
	FullName:  TimeEx::FormatDataToWstr
	Returns:   std::wstring
	Description:     
	Parameter: LPCTSTR fmt
	Parameter: DWORD const -
	Parameter: size_t const sz -  ,   128 
	************************************************************************/
	std::wstring fn_FormatDataToWstr(LPCTSTR fmt, DWORD const data, size_t const sz = 128);
};



#endif //__SNTP_H__