
#ifndef XTIME_H
#define XTIME_H

#pragma once
#include <stdint.h>

static wstring fn_FormatDataToWstr(LPCTSTR fmt, DWORD const data, size_t const sz = 128);
uint32_t xtmtot(struct XTM *t);
void xttotm(struct XTM *t, uint32_t secsarg);
uint32_t xtConvertSysTimeToUxTime(SYSTEMTIME & systm);

struct XTM
{       /* date and time components */
	uint8_t		tm_sec;
	uint8_t		tm_min;
	uint8_t		tm_hour; /* 0-23 */
	uint8_t		tm_mday; /* 1-31 */
	uint8_t		tm_wday; /* 0  6; Sunday = 0 */
	uint8_t		tm_mon;  /* 1-12 */
	uint16_t	tm_year; /* 19xx, 20xx*/
};

#define	_TBIAS_YEAR		1900
#define DAY_IN_YEAR		365
#define SEC_IN_DAY		86400
#define SEC_IN_WEAK		604800
#define SEC_IN_HOUR		3600
#define SEC_IN_MIN		60

#define _TBIAS_DAYS		((70 * (uint32_t)DAY_IN_YEAR) + 17)
#define _TBIAS_SECS		(_TBIAS_DAYS * (uint32_t)SEC_IN_DAY)
#define MONTAB(year)	((((year) & 03) || ((year) == 0)) ? mos : lmos)
#define	Daysto32(year, mon)	(((year - 1) / 4) + MONTAB(year)[mon])


uint16_t const lmos[] = { 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335 };
uint16_t const mos[] = { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };


/***********************************************************************
 FullName:  xtmtot
 Returns:   uint32_t
 Description: calendar to timer
 Parameter: struct XTM * t
 ************************************************************************/
uint32_t xtmtot(struct XTM *t)
{	/* convert time structure to scalar time */
	int32_t		days;
	uint32_t	secs;
	int32_t		mon, year;

	/* Calculate number of days. */
	mon = t->tm_mon - 1;
	year = t->tm_year - _TBIAS_YEAR;
	days = Daysto32(year, mon) - 1;
	days += DAY_IN_YEAR * year;
	days += t->tm_mday;
	days -= _TBIAS_DAYS;

	/* Calculate number of seconds. */
	secs = SEC_IN_HOUR * t->tm_hour;
	secs += 60 * t->tm_min;
	secs += t->tm_sec;

	secs += (days * (uint32_t)SEC_IN_DAY);

	return (secs);
}


/***********************************************************************
 FullName:  xttotm
 Returns:   void
 Description: timer to calendar
 Parameter: struct XTM * t
 Parameter: uint32_t secsarg
 ************************************************************************/
void xttotm(struct XTM *t, uint32_t secsarg)
{
	uint32_t	secs;
	int32_t		days;
	int32_t		mon;
	int32_t		year;
	int32_t		i;
	const uint16_t*	pm;

	if (secsarg >= 0)
	{
		secs = (uint32_t)secsarg;
		days = _TBIAS_DAYS;
	}
	else
	{
		secs = (uint32_t)secsarg + _TBIAS_SECS;
		days = 0;
	}

	/* days, hour, min, sec, day of weak */
	days += secs / SEC_IN_DAY;		
	secs = secs % SEC_IN_DAY;
	t->tm_hour = (uint8_t)(secs / SEC_IN_HOUR);	
	secs %= SEC_IN_HOUR;
	t->tm_min = (uint8_t)(secs / SEC_IN_MIN);		
	t->tm_sec = secs % SEC_IN_MIN;
	t->tm_wday = ((secsarg - 345600) % SEC_IN_WEAK) / SEC_IN_DAY;

	/* determine year */
	for (year = days / DAY_IN_YEAR; days < (i = Daysto32(year, 0) + DAY_IN_YEAR * year);) 
	{ --year; }
	days -= i;
	t->tm_year = (uint16_t)(year + _TBIAS_YEAR);

	/* determine month */
	pm = MONTAB(year);
	for (mon = 12; days < pm[--mon];)
		;
	t->tm_mon = (uint8_t)(mon + 1);
	t->tm_mday = (uint8_t)(days - pm[mon] + 1);
}

uint32_t xt_diff_tm(uint32_t tmold, uint32_t tmnew)
{
	return (uint32_t)abs ((int32_t)(tmnew - tmold));
}

std::wstring xtFormatUXtimeToWstr(XTM& utm /*IN*/, BOOL shortStr = 0)
{
	UINT DayOfWeek[] =
	{
		LOCALE_SDAYNAME1,
		LOCALE_SDAYNAME2,
		LOCALE_SDAYNAME3,
		LOCALE_SDAYNAME4,
		LOCALE_SDAYNAME5,
		LOCALE_SDAYNAME6,   // Saturday
		LOCALE_SDAYNAME7,   // Sunday
	};

	UINT Month[] =
	{
		LOCALE_SMONTHNAME1,
		LOCALE_SMONTHNAME2,
		LOCALE_SMONTHNAME3,
		LOCALE_SMONTHNAME4,
		LOCALE_SMONTHNAME5,
		LOCALE_SMONTHNAME6,
		LOCALE_SMONTHNAME7,
		LOCALE_SMONTHNAME8,
		LOCALE_SMONTHNAME9,
		LOCALE_SMONTHNAME10,
		LOCALE_SMONTHNAME11,
		LOCALE_SMONTHNAME12,
	};

	WCHAR buf[64];
	::GetLocaleInfo(MAKELCID(LANG_RUSSIAN, LANG_NEUTRAL),	// Get string for day 
		DayOfWeek[utm.tm_wday],		// Get day of week 
		buf, sizeof(buf) / sizeof(WCHAR));
	wstring dayOfWeek(buf);

	::GetLocaleInfo(MAKELCID(LANG_RUSSIAN, LANG_NEUTRAL),   // Get string for month 
		Month[utm.tm_mon - 1],// Get month    
		buf,
		sizeof(buf) / sizeof(buf[0]));
	wstring month(buf);

	wstring day = fn_FormatDataToWstr(L"%2.2d", utm.tm_mday);
	wstring year = fn_FormatDataToWstr(L"%4.4d", utm.tm_year);
	wstring hour = fn_FormatDataToWstr(L"%2.2d", utm.tm_hour);
	wstring minute = fn_FormatDataToWstr(L"%2.2d", utm.tm_min);
	wstring sec = fn_FormatDataToWstr(L"%2.2d", utm.tm_sec);

	if (!shortStr)
	{
		day += L" ";
		day += month + L" ";
		day += year + L", ";
		day += hour + L":" + minute + L":" + sec + L", " + dayOfWeek + L", UTC";
	}
	else
	{
		day += L"/" + fn_FormatDataToWstr(L"%2.2d", utm.tm_mon) + L"/" + year + L" " + hour + L":"
			+ minute + L":" + sec;
	}

	return (day += L"\r\n");
}

/***********************************************************************
FullName:  xtConvertSysTimeToUxTime
Returns:  
Description:  SYSTEMTIME UTC  unix time UTC 
Parameter: SYSTEMTIME & systm

Note:
tm_hour
Hours since midnight (023)
tm_mday
Day of month (131)
tm_min
Minutes after hour (059)
tm_mon
Month (011; January = 0)
tm_sec
Seconds after minute (059)
tm_year
Year (current year)
************************************************************************/
uint32_t xtConvertSysTimeToUxTime(SYSTEMTIME & systm)
{
	XTM timeT;
	timeT.tm_wday = (uint8_t) systm.wDayOfWeek;
	timeT.tm_mon = (uint8_t) systm.wMonth;
	timeT.tm_mday = (uint8_t) systm.wDay;
	timeT.tm_year = (uint16_t) systm.wYear;
	timeT.tm_hour = (uint8_t) systm.wHour;
	timeT.tm_min = (uint8_t) systm.wMinute;
	timeT.tm_sec = (uint8_t) systm.wSecond;

	return xtmtot(&timeT);
}


/***********************************************************************
FullName:  TimeEx::FormatDataToWstr
Returns:   std::wstring
Description:     
Parameter: LPCTSTR fmt
Parameter: DWORD const -
Parameter: size_t const sz -  ,   128 
************************************************************************/
std::wstring fn_FormatDataToWstr(LPCTSTR fmt, DWORD const data, size_t const sz /*= 128*/)
{
	wstring ws;
	ws.resize(sz);
	swprintf_s(const_cast<wchar_t*>(ws.c_str()), sz, fmt, data);
	return wstring(ws.c_str());
}


#endif



